# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.initial_control_state import InitialControlState

from .schema_helper import SchemaHelper
from .models_impl import *


class RotorInPowerProduction(InitialControlState):
    r"""
    The rotor begins the simulation in the power production state.  This is the default behaviour, and this initial condition is not required to be present, although it can be added to make it explicit.
    
    Attributes
    ----------
    InitialConditionType : Literal['RotorInPowerProduction'], default='RotorInPowerProduction'
        Defines the specific type of InitialCondition model in use.  For a `RotorInPowerProduction` object, this must always be set to a value of `RotorInPowerProduction`.
    
    Notes
    -----
    
    """
    InitialConditionType: Literal['RotorInPowerProduction'] = Field(alias="InitialConditionType", default='RotorInPowerProduction', allow_mutation=False, const=True) # type: ignore

    _relative_schema_path = 'TimeDomainSimulation/InitialCondition/RotorInPowerProduction.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'InitialConditionType').merge(InitialControlState._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


RotorInPowerProduction.update_forward_refs()
