# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.dynamic_wake import DynamicWake

from .schema_helper import SchemaHelper
from .models_impl import *


class EquilibriumWakeModel(DynamicWake):
    r"""
    The equilibrium wake model instantaneously updates the induction in response to changes operating condition.  This represents the steady state solution.  It should not be used in linearisation as it would represent an instantaneous response of the wake, where in reality the response is significantly lagged.  It is also not a good approach for time domain as the lagged wake response is not captured.
    
    Attributes
    ----------
    DynamicWakeType : Literal['EquilibriumWakeModel'], default='EquilibriumWakeModel'
        Defines the specific type of DynamicWake model in use.  For a `EquilibriumWakeModel` object, this must always be set to a value of `EquilibriumWakeModel`.
    
    Notes
    -----
    
    """
    DynamicWakeType: Literal['EquilibriumWakeModel'] = Field(alias="DynamicWakeType", default='EquilibriumWakeModel', allow_mutation=False, const=True) # type: ignore

    _relative_schema_path = 'Settings/AerodynamicSettings/AerodynamicModel/MomentumTheoryCorrections/DynamicWake/EquilibriumWakeModel.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        'DynamicWakeType').merge(DynamicWake._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


EquilibriumWakeModel.update_forward_refs()
