# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder
from dnv_bladed_models.bladed_model import BladedModel
class AdditionalConstrainedWave_MethodEnum(str, Enum):
    OFF = "OFF"
    ON = "ON"
    NON_LINEAR = "NON-LINEAR"

from .schema_helper import SchemaHelper
from .models_impl import *


class AdditionalConstrainedWave(BladedModel):
    r"""
    The definition of additional constrained wave options.
    
    Not supported yet.
    
    Attributes
    ----------
    Method : AdditionalConstrainedWave_MethodEnum, Not supported yet
        The method used to define a constrained wave.
    
    WaveHeight : float, Not supported yet
        The constrained wave height, measured from trough to crest.
    
    Period : float, Not supported yet
        The constrained wave time period when using the Stream Function method.
    
    BlendNonLinearConstrained : bool, default=True, Not supported yet
        If true, the non-linear surface elevation will be bleded in.
    
    TimeOfWaveCrest : float, Not supported yet
        The time of the crest of the constrained wave.
    
    NumberOfConstraints : int, Not supported yet
        The number of constraints.
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    Method: AdditionalConstrainedWave_MethodEnum = Field(alias="Method", default=None) # Not supported yet
    WaveHeight: float = Field(alias="WaveHeight", default=None) # Not supported yet
    Period: float = Field(alias="Period", default=None) # Not supported yet
    BlendNonLinearConstrained: bool = Field(alias="BlendNonLinearConstrained", default=None) # Not supported yet
    TimeOfWaveCrest: float = Field(alias="TimeOfWaveCrest", default=None) # Not supported yet
    NumberOfConstraints: int = Field(alias="NumberOfConstraints", default=None) # Not supported yet

    _relative_schema_path = 'TimeDomainSimulation/Environment/SeaState/Waves/AdditionalConstrainedWave/AdditionalConstrainedWave.json'
    _type_info = TypeInfo(
        set([]),
        set([]),
        set([]),
        None).merge(BladedModel._type_info)


    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass

    def _entity(self) -> bool:
        return True


AdditionalConstrainedWave.update_forward_refs()
